'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _transcriptTurn = require('./transcript-turn');

var _transcriptTurn2 = _interopRequireDefault(_transcriptTurn);

var _functify = require('./functify');

var _functify2 = _interopRequireDefault(_functify);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TranscriptView = function (_React$Component) {
  (0, _inherits3.default)(TranscriptView, _React$Component);

  function TranscriptView(props) {
    (0, _classCallCheck3.default)(this, TranscriptView);

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(TranscriptView).call(this, props));

    _this.handleTurnMounted = _this.handleTurnMounted.bind(_this);
    _this.handleResize = (0, _utils.debounce)(_this.handleResize, 100).bind(_this);
    _this.state = { startTime: 0,
      endTime: Number.MAX_VALUE,
      nextTurnIndex: 0,
      windowDimensions: window.innerHeight + 'x' + window.innerWidth };
    return _this;
  }

  (0, _createClass3.default)(TranscriptView, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      window.addEventListener('resize', this.handleResize, false);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.time > this.state.endTime || nextProps.time < this.state.startTime) {

        this.setState({ startTime: nextProps.time,
          endTime: Number.MAX_VALUE,
          nextTurnIndex: this.findNextTurnIndex(nextProps.turns, nextProps.time)
        });
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener('resize', this.handleResize);
    }
  }, {
    key: 'findNextTurnIndex',
    value: function findNextTurnIndex(turns, time) {
      for (var i = 0; i < turns.length; i++) {
        if (turns[i].end > time) return i;
      }
      return 0;
    }
  }, {
    key: 'handleResize',
    value: function handleResize() {
      this.setState({ windowDimensions: window.innerHeight + 'x' + window.innerWidth,
        endTime: Number.MAX_VALUE,
        nextTurnIndex: this.findNextTurnIndex(this.props.turns, this.props.time)
      });
    }
  }, {
    key: 'handleTurnMounted',
    value: function handleTurnMounted(status, index, end) {
      var nextState = {};
      switch (status) {
        case 'ok':
          nextState.nextTurnIndex = index + 1;
          break;
        case 'truncated':
          nextState.endTime = end;
          break;
        case 'failed':
          nextState.endTime = this.props.turns[index - 1].end;
          nextState.nextTurnIndex = index - 1;
          break;
        default:
          throw 'unexpected status ' + status;
      }
      if (this.props.time > nextState.endTime) {
        nextState = { startTime: this.props.time,
          endTime: Number.MAX_VALUE,
          nextTurnIndex: this.findNextTurnIndex(this.props.turns, this.props.time)
        };
      }
      this.setState(nextState);
    }
  }, {
    key: 'createTurnView',
    value: function createTurnView(index, turn) {
      var _this2 = this;

      var speech = (0, _functify2.default)(turn.speech).enumerate().map(function (_ref) {
        var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

        var i = _ref2[0];
        var s = _ref2[1];
        s.index = i;return s;
      }).skipWhile(function (s) {
        return s.end < _this2.state.startTime;
      }).takeUntil(function (s) {
        return s.start > _this2.state.endTime;
      }).toArray();
      return _react2.default.createElement(_transcriptTurn2.default, {
        highlights: this.props.highlights.get(index),
        index: index,
        key: 'turn-' + index + '.' + this.state.startTime,
        onMounted: this.handleTurnMounted,
        onSpeechClick: this.props.onSeekRequest,
        progress: (0, _utils.progress)(this.props.time, turn.start, turn.end),
        sentences: turn.sentences,
        speaker: this.props.speakers[turn.speaker],
        speech: speech,
        time: this.props.time
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var turnViews = (0, _functify2.default)(this.props.turns).enumerate().skipWhile(function (_ref3) {
        var _ref4 = (0, _slicedToArray3.default)(_ref3, 2);

        var turn = _ref4[1];
        return turn.end < _this3.state.startTime;
      }).takeUntil(function (_ref5) {
        var _ref6 = (0, _slicedToArray3.default)(_ref5, 1);

        var index = _ref6[0];
        return index > _this3.state.nextTurnIndex;
      }).map(function (_ref7) {
        var _ref8 = (0, _slicedToArray3.default)(_ref7, 2);

        var index = _ref8[0];
        var turn = _ref8[1];
        return _this3.createTurnView(index, turn);
      }).toArray();
      return _react2.default.createElement(
        'div',
        {
          className: 'flex-auto',
          key: this.state.windowDimensions
        },
        turnViews
      );
    }
  }]);
  return TranscriptView;
}(_react2.default.Component);

TranscriptView.propTypes = { ended: _react2.default.PropTypes.bool,
  highlights: _react2.default.PropTypes.instanceOf(_immutable2.default.List),
  onSeekRequest: _react2.default.PropTypes.func.isRequired,
  speakers: _react2.default.PropTypes.arrayOf(_react2.default.PropTypes.string).isRequired,
  time: _react2.default.PropTypes.number.isRequired,
  turns: _react2.default.PropTypes.arrayOf(_react2.default.PropTypes.object).isRequired
};
TranscriptView.defaultProps = { ended: false,
  highlights: _immutable2.default.List.of()
};
exports.default = TranscriptView;